// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

//
// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//
function flag = assert_equal ( computed , expected )
  if ( and ( computed==expected ) ) then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

computed = specfun_gammainc(1,2);
assert_close ( computed , 0.264241117657115 , 10 * %eps );
//
computed = specfun_gammainc(2,3);
assert_close ( computed , 0.323323583816936 , 10 * %eps );
//
computed = specfun_gammainc(2,3,"lower");
assert_close ( computed , 0.323323583816936 , 10 * %eps );
//
// We have specfun_gammainc(x,a,"lower") == 1 - specfun_gammainc(x,a,"upper")
computed = specfun_gammainc(2,3,"upper");
assert_close ( computed , 0.676676416183064 , 10 * %eps );
//
// The following example shows how to use the tail argument.
// For a=1 and x>40, the result is so close to 1 that the
// result is represented by the floating point number computed=1.
computed = specfun_gammainc(40,1);
assert_close ( computed , 1 , %eps );
//
// This is why we may compute the complementary probability with
// the tail option.
computed = specfun_gammainc(40,1,"upper");
assert_close ( computed , 4.248354255291594e-018 , 10 * %eps );

//
// Show vectorized support
//
// Show expansion of a
x = [1 2 3;4 5 6];
a = 2;
computed = specfun_gammainc(x,a);
expected = [
    2.642411176571152764D-01    5.939941502901617820D-01    8.008517265285441944D-01  
    9.084218055563291205D-01    9.595723180054872570D-01    9.826487347633354741D-01  
];
assert_close ( computed , expected , 10 * %eps );
//
// Show expansion of x
x = 2;
a = [1 2 3;4 5 6];
computed = specfun_gammainc(x,a);
expected = [
    8.646647167633872977D-01    5.939941502901617820D-01    3.233235838169365439D-01  
    1.428765395014529316D-01    5.265301734371115316D-02    1.656360848061444804D-02  
];
assert_close ( computed , expected , 10 * %eps );
//
// Check error when size of a and x do not match
x = [1 2 3 4 5 6];
a = [1 2 3;4 5 6];
instr = "computed = specfun_gammainc(x,a);";
ierr = execstr(instr,"errcatch");
errmsg = lasterror();
assert_equal ( ierr , 10000 );
assert_equal ( errmsg , "specfun_gammainc: Arguments #1 and #2 do not match. Size of x is [1,6] while size of a is [2,3]." );

