// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2009 - Pascal Getreuer
//
// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->

//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//
function flag = assert_equal ( computed , expected )
  if ( and ( computed==expected ) ) then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

//
// assert_true --
//   Returns 1 if the condition is true
// Arguments
//   computed : the matrix of booleans
//
function flag = assert_true ( computed )
  if ( and ( computed ) ) then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction

// Basic test
c = specfun_lambertw([0 -exp(-1); %pi 1]);
e = [
    0                     -1.                   
    1.07365819479614921    0.56714329040978384  
];
assert_close ( c , e , %eps );

// *** Inversion Error Test ***
// Ideally, lambertw(b,z)*exp(lambertw(b,z)) = z for any complex z
// and any integer branch index b, but this is limited by machine
// precision.  The inversion error |lambertw(b,z)*exp(lambertw(b,z)) - z|
// is small but worth minding.
//   
// Experimentation finds that the error is usually on the order of 
// |z|*1e-16 on the principal branch.  This test computes the inversion
// error over the square [-10,10]x[-10,10] in the complex plane, large
// enough to characterize the error away from the branch points at
// z = 0 and -1/e.

// Use NxN points to sample the complex plane
N = 81;    
// Sample in the square [-R,R]x[-R,R]
R = 10;    
x = linspace(-R,R,N);
y = linspace(-R,R,N);
[xx,yy] = meshgrid(x,y);
z = xx + %i*yy;

for b = -4:4
  w = specfun_lambertw(b,z);
  InvError = abs(w.*exp(w) - z);
  assert_true ( max(InvError) < 1.e-13 );
end


